Param([Boolean]$add=$true)

$computername = "$env:computername.$env:userdnsdomain"
$server = $env:computername

$result = $false

Function does-regkey-exist {
	Param(
		[Microsoft.Win32.RegistryHive] $hive, 
		[String] $registryKey
	)

	$remoteKey = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey($hive, "")
	$ref = $remoteKey.OpenSubKey($registryKey);
	if (!$ref) {
		$false
	} else {
		$ref.Close()
		$true
	}
}

Function change-regkey-permissions {
	Param(
		[Microsoft.Win32.RegistryHive] $hive, 
		[String] $registryKey
	)
	
	$username = "Administrators" 
	write-output "Grant Administrators group FullControll on $registryKey ..."

	try { 
		$myAcct = whoami 
		write-output "You are $myAcct"

		$me = [Security.Principal.NTAccount]"$myAcct"
		$remoteKey = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey($hive, "")

		try {
		  # The target key is a special key, so need to modify ownership first 
		  $regKey = $remoteKey.OpenSubKey($registryKey,[Microsoft.Win32.RegistryKeyPermissionCheck]::ReadWriteSubTree,[System.Security.AccessControl.RegistryRights]::TakeOwnership)

		  # Get all access rules
		  $colAcl = $regKey.GetAccessControl()
		  
		  [System.Security.Principal.NTAccount]$ident = [System.Security.Principal.NTAccount]"$username"

		  $prevOwner = $colAcl.GetOwner([Security.Principal.NTAccount])
		  $colAcl.SetOwner($me)
		  $regKey.SetAccessControl($colAcl)
		} catch [Exception] {
		  write-output "Failed to change ownership of $registryKey on $computername to $me"
		  write-output $_.Exception.ToString()
		  write-output $false
		  return $false
		}
		write-output "Old owner is $prevOwner and new owner is $me for $registryKey on $computername."

		try {
		  # now add full permission for the target user

		  # construct the access rule 
		  $rule = New-Object System.Security.AccessControl.RegistryAccessRule ("$username","FullControl","ContainerInherit","None","Allow")
		  if ($add -eq $true) {
			$colAcl.AddAccessRule($rule)
		  } else {
			$colAcl.RemoveAccessRule($rule)
		  }
		  $colAcl.SetAccessRuleProtection($true,$true)
		  $regKey.SetAccessControl($colAcl)
		} catch [Exception] {
		  write-output "Failed to change ownership of $registryKey on $computername to $username"
		  write-output $_.Exception.ToString()
		  write-output $false
		  return $false
		}		

		# now revert ownership back to the previous owner
		#
		# LEAVE THE OWNER AS ADMINISTRATORS
		# $colAcl.SetOwner($prevOwner)
		# $regKey.SetAccessControl($colAcl)

		# close the key
		$regKey.Close()

	} catch [Exception] {
	  write-output "Failed to restore ownership of $registryKey on $computername"
	  write-output $_.Exception.ToString()
	  write-output $false
	  return $false
	}

	write-output "Updated registry key $registryKey permissions on $computername."
	return $true
}

Function try-start-service {
	Param(
		[String] $serviceName
	)

    $service = Get-Service -Name $serviceName
    if ($service.status.ToString() -eq "Running") {
      write-output "Stopping $serviceName on $server"
      $service.Stop()
      $i=0;
      while (($i -le 5) -and ($service.status.ToString() -ne "Stopped")) {
        write-output "Waiting for status change ..." $i
        Start-sleep -Seconds 1
        $service.Refresh()        
      }
      if ($i -gt 5) { write-output "Service takes too long to stop";
        return $false; 
      }
	}
	
	write-output "Current status" $service.Status
	write-output "Starting $serviceName on $server"
	$service.Start()
	$service.Refresh()
	$i=0;
	while (($i -le 5) -and ($service.status.ToString() -ne "Running")) {
	  write-output "Waiting for status change ..." $i
	  Start-sleep -Seconds 1
	  $service.Refresh()        
	}
	if ($i -gt 5) { 
	  write-output "Service takes too long to start ..."
	  return $false
	}
	write-output "Service $serviceName started on $server ..."
	return $true
}

$registryKey = "CLSID\{76A64158-CB41-11D1-8B02-00600806D9B6}"
$result = change-regkey-permissions "ClassesRoot" $registryKey

if ($result -eq $true) {
	$registryKey = "SOFTWARE\Classes\Wow6432Node\CLSID\{76A64158-CB41-11D1-8B02-00600806D9B6}"
	$exists = does-regkey-exist "LocalMachine" $registryKey
	if ($exists -eq $true) {
		$result = change-regkey-permissions "LocalMachine" $registryKey
	}
}

if ($result -eq $true) {
	$serviceName = "RemoteRegistry"
	try {
		$result = try-start-service($serviceName)
	} catch [Exception] {
		$except = $_.Exception.ToString()
		write-output "Failed to restart service $serviceName on $server ... "
		write-output $except
		if ($except -like "*System.ComponentModel.Win32Exception: The service cannot be started, either because it is disabled or because it has no enabled devices associated with it*")
		{
			try {
				write-output "The service is disabled ... changing to Automatic start mode."
				(gwmi win32_service -computername "localhost" -filter "name='RemoteRegistry'").ChangeStartMode("Automatic")
				write-output "Trying to start service again."
				$result = try-start-service($serviceName)			
			} catch [Exception] {
				$except = $_.Exception.ToString()
				write-output "Failed to restart service $serviceName on $server ... "
				write-output $except
			}			
		}
		
		write-output $result
		return
	}
}

write-output $result
