$computername = "$env:computername.$env:userdnsdomain"

$result = $false

write-output "Check Firewall Settings for WMI ..."

$isEarlyWin = $false
if ([environment]::osversion.version.major -le 5)
{
	write-output "Using old netsh calls since this is <= 5.2 (Win2003, XP, 2000) build ..."
	$isEarlyWin = $true
}

Function IsFirewallEnabled {
	Param([String]$Profile)

	$HKLM = 2147483650

	$reg = get-wmiobject -list -namespace root\default | where-object { $_.name -eq "StdRegProv" }
	$firewallEnabled = $reg.GetDwordValue($HKLM, "System\ControlSet001\Services\SharedAccess\Parameters\FirewallPolicy\" + $Profile + "Profile", "EnableFirewall")

	[bool]($firewallEnabled.uValue)
}
	
Function DoesExist-FirewallRule {
  Param([String]$RuleName)

	<# Need to check if this command is not supported.  Seems to be the case on earlier 2003 #>
    $output = netsh advfirewall firewall show rule name=$RuleName
    write-output "Netsh output:" + $output
	$matches = $output -match $RuleName
	
	$matches.length -ge 1
}

Function Get-FirewallRule {
    Param([String]$RuleName)

	$output = netsh advfirewall firewall show rule name=$RuleName
    try {
	  if ($output.length -le 3) {
		 $firewallRule = new-object PSObject -property @{NotFound = "Yes"}
	  } else {
		$firewallRule = new-object PSObject -property @{NotFound = "No"}
		$output = $output[3..($output.length-3)]
		foreach($line in $output) { 
			if ($line.length -eq 0) { break }
			$parts = $line -split ":\s+", 2
                        add-member -inputobject $firewallRule -name $parts[0] -value $parts[1] -membertype noteproperty
		}
	  }
	} catch [Exception] {
	    write-output "Output was:"
		write-output $output
		write-output ""
		throw $_.Exception
	}
	$firewallRule
}

Function Is-Firewall-Enabled-Older-Win {
	$out = sc.exe queryex sharedaccess
	write-output $out
	$isEnabled = $true

	foreach($line in $out) {
		if ($line.Contains("STOPPED")) { $isEnabled = $false }
	}
	write-output "Is firewall enabled: " + $isEnabled
	[bool]$isEnabled
}

<# advfirewall is not available on Win 2003 and earlier ... #>
Function Is-Firewall-Configured-Older-Win {
	$out = netsh firewall show service
	write-output $out
	$foundRule = $false

	foreach($line in $out) {
		if ($line.Contains("Remote Administration")) { $foundRule = $true }
	}
	write-output "Is firewall configured: " + $foundRule
	[bool]$foundRule
}

<# *********************** CHECK WMI FIREWALL SETTING **************************** #>
if ($isEarlyWin -eq $true)
{
	$configured = Is-Firewall-Configured-Older-Win
	$result = $configured[-1]
	
	if ($result -eq $false) {
		$enabled = Is-Firewall-Enabled-Older-Win
		$isEnabled = $enabled[-1]
		if ($isEnabled -eq $false) {
			write-output "Firewall is disabled so nothing to configure."
			$result = $true
		}
	}
}
else
{
	
	$ruleName = "Windows Management Instrumentation (WMI-Out)"
	try {
		$rule = Get-FirewallRule($ruleName)
	} catch [Exception] {
	  write-output "Failed to check $ruleName on $computername."
	  write-output $_.Exception.TOString()
	  write-output $_.Exception
	  write-output $_.Exception.ErrorDetails
	}

	if (($rule.NotFound -eq "Yes") -or ($rule.Enabled -ne "Yes")) {
	  write-output "$ruleName on $computername is not enabled."
	} elseif ($rule.Direction -ne "Out" -or $rule.Action -ne "Allow" -or $rule.localIP -ne "Any" -or $rule.remotePort 	-ne "Any") 	{
	  write-output "$ruleName on $computername does not match expected default."
	  write-output $rule
	} else {
	  $result = $true
	}

	if ($result -eq $true) {
		$result = $false
		
		<# *********************** CHECK RPC FIREWALL TRAFFIC **************************** #>
		<# Note for Amazon, the profile should be public instead of domain. #>
		$profile = "domain"
		$ruleName = "SumoLogicRPC"
		try {
			$exists = DoesExist-FirewallRule($ruleName)
			if ($exists[-1] -eq $false) {
				write-output "Failed to find $ruleName on $computername."
			} else {
			    $result = $true
			}
		} catch [Exception] {
		  write-output "Failed to check $ruleName on $computername."
		  write-output $_.Exception.ToString()
		}
	}
}

<# If we don't see the specific firewall rules, then check if the entire firewall is disabled. #>
<# This may not be quite right, in some cases it can use "Standard" profile rather than domain profile ... #>
if ($result -eq $false) {
	$firewallEnabled = IsFirewallEnabled("Domain")
		
	if ($firewallEnabled -eq $false) {
		write-output "Domain Profile Firewall is disabled so no ports are blocked."
		$result = $true
	}
}

write-output $result
