$computername = "$env:computername.$env:userdnsdomain"
$server = $env:computername

$result = $false


Function does-regkey-exist {
	Param(
		[Microsoft.Win32.RegistryHive] $hive, 
		[String] $registryKey
	)

	$remoteKey = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey($hive, "")
	$ref = $remoteKey.OpenSubKey($registryKey);
	if (!$ref) {
		$false
	} else {
		$ref.Close()
		$true
	}
}

Function check-regkey-permissions {
	Param(
		[Microsoft.Win32.RegistryHive] $hive, 
		[String] $registryKey
	)
	$found = $false
	$username = "Administrators" 
	
	try 
	{
	  write-output "Check if Adminstrators group has FullControl on $registryKey"
	
	  $remoteKey = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey($hive, "")

	  # The target key is a special key, so need to modify ownership first.
	  $regKey = $remoteKey.OpenSubKey($registryKey)

	  # Get all access rules.
	  $colAcl = $regKey.GetAccessControl()
	  $accessRules = $colAcl.GetAccessRules($true, $true, [System.Security.Principal.NTAccount])
	  
	  $expectedRegistryRights = "FullControl"
	  $matchIdent = "*" + $username.ToLower()
	  
	  $found = $false
	  foreach($accessRule in $accessRules) {
		 $ident = $accessRule.IdentityReference.ToString().ToLower()
		 $registryRights = $accessRule.RegistryRights.ToString()
	 
		 $found = ($ident -like $matchIdent) -and ($registryRights -eq $expectedRegistryRights)
		 if ($found -eq $true) { break } 
	  }
	  if ($found -eq $false) {
		write-output "Did not find expected rule."
		write-output $accessRules
	  } else {
		write-output "Found expected access rule."
	  }	
				
	  # close the key
	  $regKey.Close()
	}
	catch [Exception] 
	{
	  write-output "Failed to query registry of $registryKey on $computername to $me"
	  write-output $_.Exception.ToString()
	}

	return $found
}

$registryKey = "CLSID\{76A64158-CB41-11D1-8B02-00600806D9B6}"
$result = check-regkey-permissions "ClassesRoot" $registryKey

if ($result -eq $true) {
	$registryKey = "SOFTWARE\Classes\Wow6432Node\CLSID\{76A64158-CB41-11D1-8B02-00600806D9B6}"
	$exists = does-regkey-exist "LocalMachine"  $registryKey
	if ($exists -eq $true) {
		$result = check-regkey-permissions "LocalMachine" $registryKey
	}
}

<# TODO: This needs to be split into a separate step since it is independent. #>
$serviceName = "RemoteRegistry"
try 
{
    $service = Get-Service -Name $serviceName
    if ($service.status.ToString() -ne "Running") {
      write-output "Remote register is not running!"
	  $result = $false
	}
}
catch [Exception] 
{
  write-output "Failed to detect if remote registry service is running."
  write-output $_.Exception.ToString()
}
	  
write-output "Finished checking registry permissions."
write-output $result
