Param(
	[string] $computer="."
)

$result = $false

$doShort = $false

for ($i=0; $i -lt $args.count; $i++) {
  if ($args[$i] -eq "-short") { $doShort = $true }
}

$computername = "$env:computername.$env:userdnsdomain"

write-output ">>>>>>> Get OperatingSystem information on ${computername} ..."
$operatingSystem = Get-WmiObject -ComputerName $computer -Class Win32_OperatingSystem | select *
write-output $operatingSystem
$sessConfig = get-pssessionconfiguration
write-output $sessConfig

write-output ">>>>>>> Get details about computer make, model and ram on ${computername} ..."
$computerSystem = Get-WmiObject -ComputerName $computer -Class Win32_ComputerSystem | select *
write-output $computerSystem

write-output ">>>>>>> Get details about the processor on ${computername} ..."
$processor = Get-WmiObject -ComputerName $computer -Class Win32_Processor | select *
write-output $processor

write-output ">>>>>>> Get processes running on ${computername} ..."
$processes = Get-Process | select *
write-output $processes

write-output ">>>>>> Getting HotFixes information on ${computername} ..."
if ($doShort -eq $true) {
	$hotfixes = (get-WmiObject -ComputerName $computer -Class Win32_QuickFixEngineering -property hotfixid | select -property HotFixId | 
	   sort { $_ } | foreach-object { if (-not ($_ -match "File*")) { $_ } } | select -expand HotFixId) -join ","
} else {
	$hotfixes = get-WmiObject -ComputerName $computer -Class Win32_QuickFixEngineering | select *
}
write-output $hotfixes
write-output ""

write-output ">>>>>>> Get Available Disk Space information on ${computername} ..."
$logicalDisk = get-WmiObject -ComputerName $computer -Class Win32_LogicalDisk -Filter "DriveType=3" | select *
write-output $logicalDisk

write-output ">>>>>>> Determine the physical adapter on ${computername} ..."
if ($OS.Version -lt 6)
{
	write-output 'Running on a version of Windows less than Vista'
    $networkAdapter = Get-WmiObject -ComputerName $computer -Class Win32_NetworkAdapter -Filter "NOT PNPDeviceID LIKE 'ROOT\\%'"  | select *
}
else
{
   $networkAdapter = Get-WmiObject -ComputerName $computer -Class Win32_NetworkAdapter -Filter "NetEnabled = True AND NOT PNPDeviceID LIKE 'ROOT\\%'" | select *
}
write-output $networkAdapter

write-output ">>>>> Getting network information on ${computername} ..."
$ipconfig = ""
if ($doShort -ne $true) {
   $ipconfig = ipconfig /all
}
write-output $ipconfig
$networkAdapterConfiguration = get-WmiObject -ComputerName $computer -Class Win32_NetworkAdapterConfiguration -Filter IPEnabled="True" | select *
write-output $networkAdapterConfiguration

write-output ">>>>> IPV4 Dynamic Port Range on ${computername} ..."
netsh int ipv4 show dynamicport tcp
netsh int ipv4 show dynamicport udp

write-output ">>>>> IPV6 Dynamic Port Range on ${computername} ..."
netsh int ipv6 show dynamicport tcp
netsh int ipv6 show dynamicport udp

<# TODO - Add check if they have locally configured a more limited dynamic RPC port range:
With Registry Editor, you can modify the following parameters for RPC. The RPC Port key values discussed below are all located in the following key in the registry:
HKEY_LOCAL_MACHINE\Software\Microsoft\Rpc\Internet\ Key Data Type


Ports REG_MULTI_SZ
Specifies a set of IP port ranges consisting of either all the ports available from the Internet or all the ports not available from the Internet. Each string represents a single port or an inclusive set of ports.

For example, a single port may be represented by 5984, and a set of ports may be represented by 5000-5100. If any entries are outside the range of 0 to 65535, or if any string cannot be interpreted, the RPC runtime treats the entire configuration as invalid.
PortsInternetAvailable REG_SZ Y or N (not case-sensitive)
If Y, the ports listed in the Ports key are all the Internet-available ports on that computer. If N, the ports listed in the Ports key are all those ports that are not Internet-available.
UseInternetPorts REG_SZ ) Y or N (not case-sensitive
Specifies the system default policy.
If Y, the processes using the default will be assigned ports from the set of Internet-available ports, as defined previously.
If N, the processes using the default will be assigned ports from the set of intranet-only ports.
#>

$desktop = ""
if ($doShort -ne $true) {
	write-output ">>>>> Getting desktop information on ${computername} ..."
    $desktop = Get-WmiObject -ComputerName $computer -Class Win32_Desktop | select *
	write-output $desktop
}

$service = ""
if ($doShort -ne $true) {
	write-output ">>>>> Getting service status information on ${computername} ..."
    $service = Get-WmiObject -ComputerName $computer -Class Win32_Service | select *
	write-output $service
}

$result = $true
