@echo off
setlocal

rem this script lives under \script\ subdirectory, so we need to back up 1 level to get to the install directory
set installDir=%~sdp0\..\

set logFile="%installDir%\logs\safeRestart.log"
set sleep=%systemroot%\System32\timeout.exe /nobreak /t 
set sc=%systemroot%\System32\sc.exe
set fc=%systemroot%\System32\fc.exe

echo %date% %time% safeRestart logging to %logFile%

echo %date% %time% safeRestart invoked, installDir="%installDir%" >> %logFile%

if exist "%installDir%\wrapper.exe.new" (
    rem A new wrapper binary is downloaded from the cloud, but it might be same as existing binary.
    rem Compare the files to see if we need to swap and bounce the service
    %fc% /B "%installDir%\wrapper.exe" "%installDir%\wrapper.exe.new" > NUL
    if not errorlevel 1 (
        echo %date% %time% New wrapper executable matches existing one. Cleaning it up, no service restart needed. >> %logFile%
        del "%installDir%wrapper.exe.new" >> %logFile% 2>&1
        goto :eof
    )

    echo %date% %time% New wrapper executable detected in install directory >> %logFile%
    echo %date% %time% Waiting 20 sec before continuing. >> %logFile%
    %sleep% 20 > NUL

    rem ---- rename files ----
    echo %date% %time% Fixing wrapper file names and restarting service to migrate to new version. >> %logFile%

    echo %date% %time% del "%installDir%wrapper.exe.backup" >> %logFile%
    del "%installDir%wrapper.exe.backup" >> %logFile% 2>&1

    echo %date% %time% rename "%installDir%wrapper.exe" wrapper.exe.backup >> %logFile%
    rename "%installDir%wrapper.exe" wrapper.exe.backup >> %logFile% 2>&1

    echo %date% %time% rename "%installDir%wrapper.exe.new" wrapper.exe >> %logFile%
    rename "%installDir%wrapper.exe.new" wrapper.exe >> %logFile% 2>&1

    rem ---- bounce the collector ----
    echo %date% %time% Restarting service. >> %logFile%

    call :SERVICEACTION stop STOPPED
    echo %date% %time% Service stopped. Waiting 5 sec before starting. >> %logFile%
    %sleep% 5 > NUL

    call :SERVICEACTION start RUNNING
    echo %date% %time% Service started >> %logFile%
    goto :eof
) else (
    echo %date% %time% No new wrapper executable detected, no action needed. >> %logFile%
    goto :eof
)

:SERVICEACTION
    setlocal
    set action=%~1
    set expected=%~2
    set max_wait_sec=120

    echo %date% %time% Issuing %action% command to sumo-collector service >> %logFile%
    %sc% %action% sumo-collector >> %logFile% 2>&1
    echo %date% %time% Command sent. Waiting for updated status. >> %logFile%

    :WAIT
    for /F "tokens=3 delims=: " %%H in ('%sc% query sumo-collector ^| findstr "STATE"') do (
        set /a max_wait_sec=%max_wait_sec%-1
        echo %date% %time% Current status ^(%max_wait_sec% sec remaining^): %%H >> %logFile%
        if /I "%%H" NEQ "%expected%" (
            if "%max_wait_sec%"=="0" (
                echo %date% %time% Timed out! >> %logFile%
                exit /b 1
            )
            %sleep% 1 > NUL
            goto :WAIT
        ) else (
            goto :eof
        )
    )
