@echo off
setlocal

:: usage: secureFiles.cmd [homeDir] [runAs]
:: Make collector directory accessible only from "sumologic_collector" group members
:: Arguments: homeDir - optional, the root directory of collector installation (if not specified, use parent folder of this script)
::            runAs   - optional, the name of account who launch the collector (if not specified, use current user)

set "homeDir=%1"
set "homeDir=%homeDir:"=%"

set "runAs=%2"
set "runAs=%runAs:"=%"
set group=sumologic_collector

if [%1] == [] (
    for %%A in ("%~dp0.") do set "homeDir=%%~dpA"
)
if [%2] == [] (
    set "runAs=%USERNAME%"
)
if not exist "%homeDir%\." (
    echo Directory "%homeDir%" is not found!
    exit /b 1
)

echo Start securing files under "%homeDir%" with [%runAs%:%group%]...

net localgroup %group%
if %errorlevel% neq 0 (
    net localgroup %group% /Comment:"SumoLogic Collector Operators" /add
)

call :join_group %group% %USERNAME%
call :join_group %group% %runAs%
if %errorlevel% neq 0 (
    exit /b %errorlevel%
)

icacls "%homeDir%\." /inheritance:d
if %errorlevel% neq 0 (
    echo Fail to set folders permission; abort.
    exit /b 5
)
icacls "%homeDir%\." /remove Users
if %errorlevel% neq 0 (
    echo Fail to set folders permission; abort.
    exit /b 6
)
icacls "%homeDir%\." /grant "%group%":F
if %errorlevel% neq 0 (
    echo Fail to set folders permission; abort.
    exit /b 7
)

echo Securing files succeed.
exit /b 0

:join_group
    net localgroup %1 | findstr /i %2
    if %errorlevel% neq 0 (
        net localgroup %1 %2 /add
        if %errorlevel% neq 0 (
            echo Fail to add account %2 into group %1; abort.
            exit /b 3
        )
    )
exit /b 0
